/* -*- c++ -*- ----------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   http://lammps.sandia.gov, Sandia National Laboratories
   Steve Plimpton, sjplimp@sandia.gov

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

/* ----------------------------------------------------------------------
   Contributing author: Aram Davtyan
------------------------------------------------------------------------- */

#ifdef FIX_CLASS

FixStyle(print_debug_forces,FixPrintDebugForces)

#else

#ifndef LMP_FIX_PRINT_DEBUG_FORCES_H
#define LMP_FIX_PRINT_DEBUG_FORCES_H

#include "stdio.h"
#include "fix.h"

namespace LAMMPS_NS {

class FixPrintDebugForces : public Fix {
 public:
  FixPrintDebugForces(class LAMMPS *, int, char **);
  ~FixPrintDebugForces();
  int setmask();
  void end_of_step();
  void post_force(int);
  void init();

 private:
  int me;
  FILE *fp;
  char *filename, *header;
  int x_flag, v_flag, q_flag;
  int nstep;
  double *buf;
};

}

#endif
#endif

/* ERROR/WARNING messages:

E: Illegal ... command

Self-explanatory.  Check the input script syntax and compare to the
documentation for the command.  You can use -echo screen as a
command-line option when running LAMMPS to see the offending line.

E: Cannot open fix print file %s

The output file generated by the fix print command cannot be opened

*/
